@@ -1,6 +1,6 @@
 /*
  *  cryptmount - a utility for user-level mounting of encrypted filesystems
- *  (C)Copyright 2005-2023, RW Penney
+ *  (C)Copyright 2005-2024, RW Penney
  */
 
 /*
@@ -478,9 +478,9 @@ static int do_devsetup(const km_pw_context_t *pw_ctxt,
 int do_devshutdown(const bound_tgtdefn_t *boundtgt)
 {   const tgtdefn_t *tgt = boundtgt->tgt;
     struct stat sbuff;
-    unsigned devcnt=0;
-    dev_t *devids=NULL;
-    int eflag=ERR_NOERROR;
+    unsigned devcnt = 0;
+    dev_t *devids = NULL;
+    int eflag = ERR_NOERROR;
 
     /* Check if filing system has been configured at all: */
     if (!is_configured(tgt->ident, NULL)) {
@@ -490,18 +490,21 @@ int do_devshutdown(const bound_tgtdefn_t *boundtgt)
         goto bail_out;
     }
 
+    /* Delay to allow udev to settle */
+    millisleep(500);    /* FIXME - find more deterministic solution! */
+
     /* Find any underlying (e.g. loopback) devices for device-mapper target: */
-    udev_settle();
     (void)devmap_dependencies(tgt->ident, &devcnt, &devids);
 #ifdef DEBUG
     fprintf(stderr, "Shutting down %s [%u dependencies]\n",
             tgt->ident, devcnt);
 #endif
 
+    memset((void*)&sbuff, 0, sizeof(sbuff));
     if (stat(tgt->dev, &sbuff) != 0) {
         fprintf(stderr, _("Cannot stat \"%s\"\n"), tgt->dev);
+        sbuff.st_mode |= S_IFREG;
         eflag = ERR_BADDEVICE;
-        goto bail_out;
     }
 
     /* Remove demice-mapper target: */
@@ -509,9 +512,8 @@ int do_devshutdown(const bound_tgtdefn_t *boundtgt)
     if (eflag != ERR_NOERROR) {
         fprintf(stderr, _("Failed to remove device-mapper target \"%s\"\n"),
             tgt->ident);
-        goto bail_out;
     }
-    udev_settle();
+    await_devmap(tgt->ident, 0, 2000);
 
     /* Tidy-up any associated loopback devices: */
     if (S_ISREG(sbuff.st_mode) && devids != NULL) {
@@ -534,9 +536,9 @@ int do_devshutdown(const bound_tgtdefn_t *boundtgt)
  *  \see do_unmount(), do_devsetup(), fs_mount().
  */
 static int do_mount(const km_pw_context_t *pw_ctxt, bound_tgtdefn_t *boundtgt)
-{   const tgtdefn_t *tgt=NULL;
-    int freedev=0, eflag=ERR_NOERROR;
-    char *mntdev=NULL;
+{   const tgtdefn_t *tgt = NULL;
+    int freedev = 0, eflag = ERR_NOERROR;
+    char *mntdev = NULL;
     tgtstat_t *tstat;
 
     if (is_mounted(boundtgt->tgt)) {
@@ -549,11 +551,13 @@ static int do_mount(const km_pw_context_t *pw_ctxt, bound_tgtdefn_t *boundtgt)
     eflag = do_devsetup(pw_ctxt, boundtgt, &mntdev);
     if (eflag != ERR_NOERROR) goto bail_out;
     tgt = boundtgt->tgt;
+    await_device(mntdev, 1, 5000);
 
 #if WITH_FSCK
     if ((tgt->flags & FLG_FSCK) != 0) {
-        if (fs_check(mntdev, tgt) != ERR_NOERROR) {
-            freedev = 1; eflag = ERR_BADMOUNT;
+        const int fsck_flag = fs_check(mntdev, tgt);
+        if (fsck_flag != ERR_NOERROR) {
+            freedev = 1; eflag = fsck_flag;
             goto bail_out;
         }
     }
@@ -573,7 +577,6 @@ static int do_mount(const km_pw_context_t *pw_ctxt, bound_tgtdefn_t *boundtgt)
 
     if (freedev) {
         /* Tidy-up debris if mount failed */
-        udev_settle();
         do_devshutdown(boundtgt);
     }
     if (mntdev != NULL) free((void*)mntdev);
@@ -668,6 +671,7 @@ static int do_swapon(const km_pw_context_t *pw_ctxt, bound_tgtdefn_t *boundtgt)
     eflag = do_devsetup(pw_ctxt, boundtgt, &mntdev);
     if (eflag != ERR_NOERROR) goto bail_out;
     tgt = boundtgt->tgt;
+    await_device(mntdev, 1, 5000);
 
     if (fs_swapon(mntdev, tgt) != ERR_NOERROR) {
         freedev = 1;
@@ -691,7 +695,6 @@ static int do_swapon(const km_pw_context_t *pw_ctxt, bound_tgtdefn_t *boundtgt)
 
     if (freedev) {
         /* Tidy-up debris if swapon failed */
-        udev_settle();
         do_devshutdown(boundtgt);
     }
 
@@ -987,7 +990,6 @@ static int do_safetynet()
     old_ident = tgt->ident;
 
     /* Get list of all targets in status-file: */
-    udev_settle();
     all_tsts = get_all_tgtstatus();
 
     for (tst=all_tsts; tst!=NULL; tst=tst->nx) {
@@ -1017,7 +1019,7 @@ static int do_safetynet()
         /* Remove device-mapper device: */
         (void)devmap_dependencies(tst->ident, &devcnt, &devids);
         if (devmap_remove(tst->ident) == ERR_NOERROR) ++counts.undeviced;
-        udev_settle();
+        await_device(devname, 0, 500);
 
         /* Free any associated loopback devices: */
         if (devcnt > 0 && loop_dellist(devcnt, devids) == 0) ++counts.unlooped;
@@ -1372,7 +1374,7 @@ cmmode_t parse_options(int argc, char *argv[],
 #ifdef _GNU_SOURCE
     struct option *longopts;
 
-    longopts = (struct option*)malloc((n_options + 1) * sizeof(struct option));
+    longopts = (struct option*)calloc(n_options + 1, sizeof(struct option));
     for (i=0; i<n_options; ++i) {
         longopts[i].name =      opt_table[i].longopt;
         longopts[i].has_arg =   ((opt_table[i].flags & NEEDS_ARG)
@@ -1586,5 +1588,5 @@ int main(int argc, char *argv[])
 }
 
 /*
- *  (C)Copyright 2005-2023, RW Penney
+ *  (C)Copyright 2005-2024, RW Penney
  */
