@@ -1,6 +1,6 @@
 /*
  *  Device-mapper utilities for cryptmount
- *  (C)Copyright 2005-2023, RW Penney
+ *  (C)Copyright 2005-2024, RW Penney
  */
 
 /*
@@ -23,6 +23,7 @@
 
 #include "config.h"
 
+#include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -32,9 +33,7 @@
 #  include <sys/sysmacros.h>
 #endif
 #include <sys/types.h>
-#if HAVE_NANOSLEEP
-#  include <time.h>
-#endif
+#include <time.h>
 #if HAVE_LIBUDEV
 #  include <libudev.h>
 #endif
@@ -47,6 +46,7 @@
 
 #include "cryptmount.h"
 #include "dmutils.h"
+#include "utils.h"
 
 
 #if !HAVE_LIBUDEV
@@ -82,17 +82,19 @@ struct dm_task *devmap_prepare(int type, const char *ident)
 }
 
 
+/*! @brief Create device-mapper full pathname from target description */
 int devmap_path(char **buff, const char *ident)
-    /* create device-mapper full pathname from target description */
-{   size_t pfxlen, sfxlen;
+{   const char prefix[] = "/dev/disk/by-id/dm-name";
+    size_t bufflen;
 
-    pfxlen = strlen(dm_dir());
-    sfxlen = strlen(ident);
-    *buff = (char*)realloc((void*)(*buff), (pfxlen + sfxlen + 2));
+    // FIXME - kernel 6.9 appears prone to automatically moving devices from /dev/mapper/* into /dev/disk/by-name/dm-name-*
 
-    snprintf(*buff, (pfxlen + sfxlen + 2), "%s/%s", dm_dir(), ident);
+    bufflen = sizeof(prefix) + strlen(ident) + 1;
+    *buff = (char*)realloc((void*)(*buff), bufflen);
 
-    return (int)(pfxlen + sfxlen + 1);
+    snprintf(*buff, bufflen, "%s-%s", prefix, ident);
+
+    return (int)(bufflen - 1);
 }
 
 
@@ -143,7 +145,7 @@ int devmap_create(const char *ident, uint64_t blk0, uint64_t blklen,
 
 
 int devmap_dependencies(const char *ident, unsigned *count, dev_t **devids)
-{   struct dm_task *dmt=NULL;
+{   struct dm_task *dmt = NULL;
     struct dm_deps *deps;
     unsigned i;
     int eflag=ERR_NOERROR;
@@ -177,24 +179,25 @@ int devmap_dependencies(const char *ident, unsigned *count, dev_t **devids)
 }
 
 
+/*! @brief Remove device-mapper target and associated device */
 int devmap_remove(const char *ident)
-    /* remove device-mapper target and associated device */
-{   struct dm_task *dmt=NULL;
+{   struct dm_task *dmt = NULL;
     struct dm_info dmi;
     struct stat sbuff;
-    char *devpath=NULL;
+    char *devpath = NULL;
     int eflag = ERR_NOERROR;
 
-    /* check device-mapper target is configured & get info: */
+    /* Check device-mapper target is configured & get info: */
     if (!is_configured(ident, &dmi)) {
         eflag = ERR_BADDEVICE;
         goto bail_out;
     }
 
-    /* remove device node (below /dev?): */
+    /* Remove device node (below /dev?): */
     devmap_path(&devpath, ident);
     if (stat(devpath, &sbuff) != 0) {
-        fprintf(stderr, "unable to stat() device node\n");
+        fprintf(stderr, "unable to stat() device node for %s at %s\n",
+                ident, devpath);
         eflag = ERR_DMSETUP;
         goto bail_out;
     }
@@ -207,7 +210,7 @@ int devmap_remove(const char *ident)
         goto bail_out;
     }
 
-    /* remove device-mapper target: */
+    /* Remove device-mapper target: */
     if ((dmt = devmap_prepare(DM_DEVICE_REMOVE, ident)) == NULL) {
         fprintf(stderr, "failed to initialize device-mapper task\n");
         eflag = ERR_DMSETUP;
@@ -250,138 +253,53 @@ int is_configured(const char *ident, struct dm_info *dminfo)
 }
 
 
-int udev_settle()
-    /*! Allow time for udev events to be processed */
-{   double totdelay = 0.0;
-    time_t starttime;
-#if HAVE_LIBUDEV
-    struct udev *udev_ctx;
-    struct udev_queue *udev_qu;
-#else
-    struct udev_queue_loc *udev_mode;
+int await_device(const char *path, int present, unsigned timeout_ms)
+    /*! Repeatedly check for presence (or absence) of block device */
+{   int st = -1, t_waited = 0, resolved = 0;
     struct stat sbuff;
-#endif
-#if HAVE_NANOSLEEP
-    struct timespec delay;
-#endif
-    int inc_ms = 250, settling = 1;
-    const double timeout = 10.0;
+    struct timespec start_time, now;
 
-    /* This routine mitigates apparent race-conditions
-     * between udev events which may temporarily take ownership of
-     * and rename newly created devices, thereby causing
-     * other processes to fail if they try to destroy
-     * or reconfigure those devices at the same time.
-     * Whether this is the responsibilty of kernel-level functions
-     * to resolve, or for user applications to mitigate,
-     * is open to debate. */
+    clock_gettime(CLOCK_REALTIME, &start_time);
 
-    time(&starttime);
-
-#if HAVE_LIBUDEV
-    udev_ctx = udev_new();
-    //udev_selinux_init(udev_ctx);
-    udev_qu = udev_queue_new(udev_ctx);
-#else
-    /* Try to find location and type of udev event queue: */
-    udev_mode = udev_queue_locations;
-    while (udev_mode->is_file) {
-        if (stat(udev_mode->path, &sbuff) == 0) break;
-        ++udev_mode;
-    }
-#endif
-
-    /* Keep waiting until there are no more queued udev events: */
     do {
-#if HAVE_NANOSLEEP
-        delay.tv_sec = inc_ms / 1000;
-        delay.tv_nsec = (inc_ms % 1000) * 1000 * 1000;
-        nanosleep(&delay, NULL);
-#else
-        sleep((unsigned)ceil(inc_ms * 1e-3));
-#endif
-        totdelay += inc_ms * 1e-3;
-        inc_ms += inc_ms / 3;
-
-#if HAVE_LIBUDEV
-        settling = !udev_queue_get_queue_is_empty(udev_qu);
-#else
-        settling = 0;
-        if (udev_mode->is_file) {
-            /* Current versions of udev place events in a single file: */
-            settling |= (udev_queue_size(udev_mode->path) > 0);
+        st = stat(path, &sbuff);
+        if (present) {
+          resolved = (st == 0 && (sbuff.st_mode & S_IFMT) == S_IFBLK);
         } else {
-            /* Older versions of udev use a directory of event files: */
-            settling |= udev_active_dir(udev_mode->path, starttime, timeout);
+          resolved = (st != 0 && errno == ENOENT);
         }
-#endif  /* HAVE_LIBUDEV */
-    } while (settling && totdelay < timeout);
 
-#if HAVE_LIBUDEV
-    udev_queue_unref(udev_qu);
-    //udev_selinux_exit(udev_ctx);
-    udev_unref(udev_ctx);
-#endif
-
-    return settling;
-}
-
-
-#if !HAVE_LIBUDEV
-
-int udev_queue_size(const char *path)
-    /*! Count number of unprocessed udev events in queue.bin file */
-{   FILE *fp;
-    unsigned long long seqnum;
-    unsigned short skiplen;
-    int nqueued = 0;
-
-    fp = fopen(path, "rb");
-    if (fp == NULL) return 0;
-    if (fread((void*)&seqnum, sizeof(seqnum), (size_t)1, fp) != 1) goto bail_out;
-
-    for (;;) {
-        skiplen = 0;
-        if (fread((void*)&seqnum, sizeof(seqnum), (size_t)1, fp) != 1
-          || fread((void*)&skiplen, sizeof(skiplen), (size_t)1, fp) != 1) break;
-
-        if (skiplen > 0) {
-            void *buff = malloc((size_t)skiplen);
-            nqueued += (fread(buff, (size_t)skiplen, (size_t)1, fp) == 1);
-            free(buff);
-        } else {
-            --nqueued;
+        if (!resolved) {
+            millisleep(250);
         }
+
+        clock_gettime(CLOCK_REALTIME, &now);
+        t_waited = (now.tv_sec - start_time.tv_sec) * 1000 +
+                    (now.tv_nsec - start_time.tv_nsec) / 1000000;
+    } while (!resolved && (t_waited < timeout_ms));
+
+    if (t_waited >= timeout_ms) {
+        fprintf(stderr, "Timeout in await_device(%s, %d, %u)\n",
+                path, present, timeout_ms);
     }
 
-  bail_out:
-    fclose(fp);
-
-    return nqueued;
+    return (resolved ? 0 : 1);
 }
 
 
-/*!
- *  Check whether the udev queue directory (e.g. /dev/.udev/queue)
- *  has been recently modified.
- *  This is only relevant to older versions of udev (e.g. 0.105).
- */
-int udev_active_dir(const char *path, time_t starttime, double timeout)
-{   struct stat sbuff;
-    int settling = 0;
+int await_devmap(const char *ident, int present, unsigned timeout_ms)
+    /*! Wrapper around await_device(), using device-mapper target name */
+{   char *tgt = NULL;
+    int status = -1;
 
-    /*  If the event directory exists, then we either have active
-        events, or possibly it is a remnant from an old udev process. */
+    devmap_path(&tgt, ident);
+    status = await_device(tgt, present, timeout_ms);
+    free((void*)tgt);
 
-    if (stat(path, &sbuff) == 0) {
-        settling |= ((starttime - sbuff.st_mtime) < 100 * timeout);
-    }
-
-    return settling;
+    return status;
 }
 
-#endif  // !HAVE_LIBUDEV
 
 /*
- *  (C)Copyright 2005-2023, RW Penney
+ *  (C)Copyright 2005-2024, RW Penney
  */
