@@ -1,6 +1,6 @@
 /*
  *  Declarations for device-mapper utilities for cryptmount
- *  (C)Copyright 2005-2023, RW Penney
+ *  (C)Copyright 2005-2024, RW Penney
  */
 
 /*
@@ -31,7 +31,6 @@
 /*! \addtogroup dev_mapper
  *  @{ */
 
-
 struct dm_task *devmap_prepare(int type, const char *devname);
 int devmap_path(char **buff, const char *ident);
 int devmap_create(const char *ident, uint64_t blk0, uint64_t blklen,
@@ -40,12 +39,13 @@ int devmap_dependencies(const char *ident, unsigned *count, dev_t **devids);
 int devmap_remove(const char *ident);
 
 int is_configured(const char *ident, struct dm_info *dminfo);
-int udev_settle();
+int await_device(const char *path, int present, unsigned timeout_ms);
+int await_devmap(const char *ident, int present, unsigned timeout_ms);
 
 /**  @} */
 
 #endif  /* _DMUTILS_H */
 
 /*
- *  (C)Copyright 2005-2023, RW Penney
+ *  (C)Copyright 2005-2024, RW Penney
  */
