@@ -1,6 +1,6 @@
 /*
  *  Miscellaneous utility functions for cryptmount
- *  (C)Copyright 2005-2023, RW Penney
+ *  (C)Copyright 2005-2024, RW Penney
  */
 
 /*
@@ -38,6 +38,7 @@
 #if HAVE_TERMIOS
 #  include <termios.h>
 #endif
+#include <time.h>
 #include <unistd.h>
 
 #include "cryptmount.h"
@@ -365,6 +366,19 @@ void sec_free(void *ptr)
 }
 
 
+void millisleep(unsigned ms)
+{
+#if HAVE_NANOSLEEP
+    struct timespec delay;
+    delay.tv_sec = ms / 1000;
+    delay.tv_nsec = (ms % 1000) * 1000 * 1000;
+    nanosleep(&delay, NULL);
+#else
+    sleep((ms + 999) / 1000);
+#endif
+}
+
+
 /**
  *  Generate a random sequence of \a len bytes,
  *  using a cryptographic-quality iterative message-digest
@@ -1035,5 +1049,5 @@ int ut_pwfort()
 #endif  /* TESTING */
 
 /*
- *  (C)Copyright 2005-2023, RW Penney
+ *  (C)Copyright 2005-2024, RW Penney
  */
